/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Zend Technologies
 *******************************************************************************/
package org.eclipse.php.internal.ui.dnd;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.php.internal.ui.Logger;
import org.eclipse.php.internal.ui.util.EditorUtility;
import org.eclipse.swt.dnd.*;
import org.eclipse.ui.PlatformUI;

/**
 * Adapter to handle file drop from other applications.
 */
public class ExternalFileDropAdapter extends DropTargetAdapter {

	private static ExternalFileDropAdapter instance;

	private ExternalFileDropAdapter() {
	}

	public static ExternalFileDropAdapter getInstance() {
		if (instance == null)
			instance = new ExternalFileDropAdapter();
		return instance;
	}

	public Transfer getTransfer() {
		return FileTransfer.getInstance();
	}

	public void dragEnter(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	}

	public void dragOperationChanged(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	}

	public void dragOver(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
	}

	public void drop(final DropTargetEvent event) {
		if (event.data instanceof String[]) {
			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
				public void run() {
					String[] files = (String[]) event.data;
					for (String file : files) {
						try {
							EditorUtility.openLocalFile(file, 0);
						} catch (CoreException e) {
							Logger.logException(e);
						}
					}
				}
			});
		}
	}
}
